package jp.co.asterisk.asreader.l251g;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.PowerManager;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.Window;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import jp.co.asterisk.asreader.l251g.view.BarcodeDemoActivity;
import jp.co.asterisk.asreader.l251g.view.BarcodeOptionActivity;
import jp.co.asterisk.asreader.l251g.view.InventoryActivity;
import jp.co.asterisk.asreader.l251g.view.OptionActivity;
import jp.co.asterisk.asreader.l251g.view.ReadMemoryActivity;
import jp.co.asterisk.asreader.l251g.view.TagAccessActivity;
import jp.co.asterisk.asreader.l251g.view.WriteMemoryActivity;

import com.asreader.lib.AsReader;
import com.asreader.lib.AsReaderManager;
import com.asreader.lib.barcode.BarcodeType;
import com.asreader.lib.device.Device;
import com.asreader.lib.device.DeviceUsbCdc;
import com.asreader.lib.device.type.ConnectionState;
import com.asreader.lib.device.type.ExceptionCode;
import com.asreader.lib.device.type.KeyState;
import com.asreader.lib.device.type.KeyType;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.R2000.R2000Model;
import com.asreader.lib.rfid.event.AsReaderEventListener;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.type.ActionState;
import com.asreader.lib.rfid.type.CommandType;
import com.asreader.lib.rfid.type.MaskType;
import com.asreader.lib.rfid.type.ResultCode;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.util.UserConfigs;
import jp.co.asterisk.asreader.l251g.support.view.base.AccessActivity;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

public class MainActivity extends Activity implements OnClickListener, OnTouchListener, AsReaderEventListener {

	private static final String TAG = MainActivity.class.getSimpleName();

	private static boolean WRITE_LOG = true;
	private static final String LOG_PATH = "Log";
	private static final String LOG_PREFIX = "ASR_L251G_Demo";

	public static final String APP_NAME = "rfid.reader.demo";
	private static final String KEY_BATTERY_INTERVAL = "battery_interval";
	private static final String KEY_MASK_TYPE = "mask_type";
	private static final int DEFAULT_BATTERY_INTERVAL = 10;
	private static final int DEFAULT_MASK_TYPE = 0;
	private static final int VIEW_INVENTORY = 1000;
	private static final int VIEW_OPTION = 1006;
	private static final int VIEW_BARCODE_DEMO = 1007;
	private static final int VIEW_BARCODE_OPTION = 1008;
	private static final int RESULT_DISCONNECTED = Activity.RESULT_FIRST_USER + 1;

	private static final int REQUEST_PERMISSION_CONTACTS = 1000;

	private PowerManager.WakeLock wakeLock = null;
	private AsReader mReader = null;

	private TextView txtDemoVersion;
	private TextView txtSDKVersion;
	private TextView txtFirmwareVersion;
	private TextView txtAddress;
	private TextView txtGlobalBand;
	private TextView txtSerialNumber;
	private Button btnInventory;
	private Button btnReadMemory;
	private Button btnWriteMemory;
	private Button btnTagAccess;
	private Button btnOption;
	private Button btnBarcodeDemo;
	private Button btnBarcodeOption;
	private Button btnLedTest;
	private Button btnRfmSensor;
	private ImageView imgBattery;
	private ImageView imgLogo;

	//private String mDeviceAddress;
	private int mBatteryInterval;
	private boolean mKeyAction;
	private MenuItem mnuConnectDevice;
	private MenuItem mnuDisconnectDevice;
	private Timer timerCheckBattery;
	private Handler mHandler;
	private Device mDevice = null;
	private MaskType mMaskType;
	
	private final int COLOR_ENABLED = Color.BLACK;
	private final int COLOR_DISABLED = Color.LTGRAY;

	private boolean mIsAlert =true;
	private String[] mPermissions = null;
	public static UserConfigs mUserConfigs = null;

	@SuppressWarnings("deprecation")
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		setProgressBarIndeterminateVisibility(true);
		setContentView(R.layout.activity_main);

		if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU){
			mPermissions = new String[] { Manifest.permission.WAKE_LOCK,
					Manifest.permission.READ_MEDIA_VIDEO,
					Manifest.permission.READ_MEDIA_IMAGES,
					Manifest.permission.READ_MEDIA_AUDIO };
		} else {
					mPermissions = new String[] { Manifest.permission.WAKE_LOCK,
							Manifest.permission.WRITE_EXTERNAL_STORAGE,
                            Manifest.permission.READ_EXTERNAL_STORAGE };
		}

		mUserConfigs = new UserConfigs(this.getApplicationContext());

		// Initialize Widget
		initWidgets();

		// Disable buttons
		enableMenuButtons(false);

		checkPermission();
	}

	private boolean checkPermission() {
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
			onPermissionsResult(true);
			return false;
		}

		ArrayList<String> permissions = new ArrayList<String>();
		int grantResult = PackageManager.PERMISSION_GRANTED;

		for (int i = 0; i < mPermissions.length; i++) {
			grantResult = PackageManager.PERMISSION_GRANTED;
			grantResult = ContextCompat.checkSelfPermission(this, mPermissions[i]);
			if (grantResult == PackageManager.PERMISSION_DENIED) {
				permissions.add(mPermissions[i]);
			}
		}
		if (permissions.size() <= 0) {
			onPermissionsResult(true);
			return false;
		}

		String[] requestPermissions = new String[permissions.size()];
		permissions.toArray(requestPermissions);
		ActivityCompat.requestPermissions(this, permissions.toArray(requestPermissions), REQUEST_PERMISSION_CONTACTS);

		ASRLog.e(TAG, "INFO. checkPermission()");
		return true;
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		if (requestCode != REQUEST_PERMISSION_CONTACTS)
			return;

		boolean result = true;
		for (int i = 0; i < grantResults.length; i++) {
			result &= grantResults[i] != PackageManager.PERMISSION_DENIED;
		}

		onPermissionsResult(result);
	}

	private void onPermissionsResult(boolean result)
	{
		if(!result) {
			ASRLog.e(TAG, "Failed to get permissions");
			return;
		}

		if(WRITE_LOG)
			ASRLog.startUp(LOG_PATH, LOG_PREFIX);

		// Using higher levels can cause serious performance degradation.
		ASRLog.setLogLevel(ASRLog.ERROR);

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Setup Always Wakeup
		setupAlwaysWakeup();

		// Load Configuration
		loadConfig();

		// Initialize RFID Reader
		initAsReader();

		// Display Demo Version
		displayVersion();

		setProgressBarIndeterminateVisibility(false);
	}

	@Override
	protected void onPause() {
		if(mReader != null) {
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onPause()");
			saveConfig();
			stopCheckBattery();
		}
		super.onPause();
	}

	@Override
	protected void onResume() {
		if(mReader != null) {
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onResume()");
			// Process to display the 'sleep' alert on the main activity.
			mReader.removeEventListener(this);
			mReader.setEventListener(this);

			R2000Model.getInstance().isTriggerEnable = false;

			if (mReader.getState() == ConnectionState.Connected) {
				startCheckBattery();
			}
		}
		super.onResume();
	}

	@Override
	protected void onStop() {
		if(mReader != null) {
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onStop()");
		}
		super.onStop();
	}
	
	@Override
	protected void onDestroy() {
		if(mReader != null) {
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

			saveConfig();
			if (timerCheckBattery != null) {
				timerCheckBattery.cancel();
				timerCheckBattery = null;
			}

			// Destroy Rfid Reader
			AsReaderManager.onDestroy();

			// Destroy USB device
			if(mDevice != null) {
				mDevice.destroy();
				mDevice = null;
			}

			defaultWakeup();

			ASRLog.shutdown();
		}
		super.onDestroy();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.INFO, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		switch (requestCode) {
		case VIEW_INVENTORY:
		case AccessActivity.VIEW_READ_MEMORY:
		case AccessActivity.VIEW_WRITE_MEMORY:
		case AccessActivity.VIEW_TAG_ACCESS:
		case VIEW_BARCODE_DEMO:
		case VIEW_BARCODE_OPTION:
			switch (resultCode) {
			case RESULT_DISCONNECTED:
				setDisconnectedState();
				break;
			default:
				break;
			}
			break;
		case VIEW_OPTION:
			switch (resultCode) {
			case RESULT_DISCONNECTED:
				setDisconnectedState();
				break;
			case RESULT_OK:
				if (data != null) {
					mBatteryInterval = data.getIntExtra(KEY_BATTERY_INTERVAL, DEFAULT_BATTERY_INTERVAL);
					int code = data.getIntExtra(KEY_MASK_TYPE, DEFAULT_MASK_TYPE);
					mMaskType = MaskType.valueOf(code);
				}
				break;
			}
			break;
		}

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateOptionsMenu()");
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {

		if(mReader != null) {
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onPrepareOptionsMenu()");
			mnuConnectDevice = menu.findItem(R.id.menu_connect_device);
			mnuDisconnectDevice = menu.findItem(R.id.menu_disconnect);

			visibleMenuItem(mReader.getState());
		}
		return super.onPrepareOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		int id = item.getItemId();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onOptionsItemSelected(%s)", getMenuId(id));

		switch (id) {
		case R.id.menu_connect_device:
			if(mDevice == null) {
				mDevice = new DeviceUsbCdc(this);
			}
			checkPermission();
			mReader.connectDevice(mDevice);
			return true;
		case R.id.menu_disconnect:
			// Disconnect Device
			stopCheckBattery();
			mReader.disconnectDevice();
			WaitDialog.show(this, R.string.disconnecting_reader);
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	public void onClick(View v) {
		Intent intent;
		int id = v.getId();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", getOnClickId(id));
		
		switch (id) {
		case R.id.inventory:
			intent = new Intent(this, InventoryActivity.class);
			intent.putExtra(KEY_MASK_TYPE, mMaskType.getCode());
			startActivityForResult(intent, VIEW_INVENTORY);
			break;
		case R.id.read_memory:
			intent = new Intent(this, ReadMemoryActivity.class);
			intent.putExtra(KEY_MASK_TYPE, mMaskType.getCode());
			startActivityForResult(intent, AccessActivity.VIEW_READ_MEMORY);
			break;
		case R.id.write_memory:
			intent = new Intent(this, WriteMemoryActivity.class);
			intent.putExtra(KEY_MASK_TYPE, mMaskType.getCode());
			startActivityForResult(intent, AccessActivity.VIEW_WRITE_MEMORY);
			break;
		case R.id.tag_access:
			intent = new Intent(this, TagAccessActivity.class);
			intent.putExtra(KEY_MASK_TYPE, mMaskType.getCode());
			startActivityForResult(intent, AccessActivity.VIEW_TAG_ACCESS);
			break;
		case R.id.option:
			intent = new Intent(this, OptionActivity.class);
			intent.putExtra(KEY_BATTERY_INTERVAL, mBatteryInterval);
			intent.putExtra(RfidActivity.KEY_ACTION, mKeyAction);
			startActivityForResult(intent, VIEW_OPTION);
			break;
		case R.id.barcode_demo:
			intent = new Intent(this, BarcodeDemoActivity.class);
			startActivityForResult(intent, VIEW_BARCODE_DEMO);
			break;
		case R.id.barcode_option:
			intent = new Intent(this, BarcodeOptionActivity.class);
			startActivityForResult(intent, VIEW_BARCODE_OPTION);
			break;
		case R.id.led_test:
			showLEDControlDialog();
			break;
		default:
			break;
		}
	}

	@SuppressWarnings("deprecation")
	@Override
	public void onStateChanged(AsReader reader, final ConnectionState state) {

		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onConnectionStateChanged(%s)", state);;

		switch (state) {
		case Disconnected:
			if (mReader.getResultCode() == ResultCode.NotSupportFirmware) {
				AlertDialog.Builder alert = new AlertDialog.Builder(this);
				alert.setTitle(R.string.system_alert);
				alert.setIcon(android.R.drawable.ic_dialog_alert);
				alert.setMessage(R.string.not_support_firmware);
				alert.setPositiveButton(R.string.action_ok, null);
				alert.show();
			}
			setDisconnectedState();
			setProgressBarIndeterminateVisibility(false);
			break;
		case Connecting:
			setProgressBarIndeterminateVisibility(true);
		case Listen:
			enableMenuButtons(false);
			imgLogo.setImageResource(R.drawable.asreader_log_off);
			break;
		case Connected:
			new Thread(new Runnable() {
				@Override
				public void run() {
					String versionName;
					String serialNumber;
					boolean rfEnabled;
					String rfModuleVersion;
					String globalBand ;
					try {
						versionName = mReader.getFirmwareVersion();
						serialNumber = mReader.getSerialNumber();
						rfEnabled = mReader.isRfidEnabled();

						if( mReader.getState() != ConnectionState.Connected) {
							ASRLog.e(TAG, "ERROR. Connected Thread - exit thread. state is not connected.");
							return;
						}

						if(!mUserConfigs.Load(mReader)) {
							ASRLog.e(TAG, "ERROR. Connected Thread - failed to load mask data.");
							return;
						}

						if(rfEnabled) {
							rfModuleVersion = mReader.getRFModuleVersion();
							globalBand = mReader.getGlobalBand().toString();

							mMaskType = mUserConfigs.getMaskType();

							// Reset previously saved settings to the reader.
							try {
								mReader.setPowerGain(mUserConfigs.getPowerGain());
								mReader.setReportRSSI(mUserConfigs.getReportRssi());
								mReader.setOperationTime(mUserConfigs.getOperationTime());
								mReader.setQuerySession(mUserConfigs.getSession());
								mReader.setQueryTarget(mUserConfigs.getTarget());
								mReader.setThreshold(mUserConfigs.getThreshold());
								mReader.setInventoryTime(mUserConfigs.getInventoryTime());
								mReader.setIdleTime(mUserConfigs.getIdleTime());
								mReader.setAutoOffTime(mUserConfigs.getAutoOffTime());
								mReader.setSleepTime(mUserConfigs.getSleepTime());
								mReader.setQValue(mUserConfigs.getQValue());
								mReader.setMinQ(mUserConfigs.getQMin());
								mReader.setMaxQ(mUserConfigs.getQMax());
								mReader.setMaskType(mMaskType);
								mReader.setLinkProfile(mUserConfigs.getLinkProfile());

							} catch (AsReaderException e) {
								e.printStackTrace();
							}
						} else {
							rfModuleVersion = "";
							globalBand = "";
						}
					} catch(AsReaderException ae) {
						ASRLog.e(TAG, "ERROR. Connected Thread - exit by exception.");
						return;
					}

					saveConfig();
					runOnUiThread(new InitRunnable(versionName, serialNumber, rfModuleVersion, globalBand, rfEnabled));
				}
			}).start();
			break;

			// SDK sends a message about an error or sleep event.
			case SDKEventMessage: {
				new Thread(new Runnable() {
					@Override
					public void run() {
						runOnUiThread(new Runnable(){
							@Override
							public void run() {
								if (mIsAlert) {
									showDialog(mReader.getSDKMessage());
									mIsAlert = false;
								}
							}
						});
					}
				}).start();
			}
			break;
		default:
			break;
			
		}
	}


	@Override
	public void onActionChanged(AsReader reader, ActionState action) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onActionChanged(%s)", action);
	}

	@Override
	public void onReadTag(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadTag(%s, [%s], %.2f, %.2f)", action, tag, rssi, phase);
	}

	@Override
	public void onReadTagWithRFM(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency, String[] rfm_data) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadedRFMTag(%s, [%s], %.2f, %.2f)", action, tag, rssi, phase);
	}

	@Override
	public void onAccessResult(AsReader reader, ResultCode code, ActionState action, String epc, String data,
			float rssi, float phase, float frequency) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s], %.2f, %.2f)", code, action, epc, data, rssi, phase);
	}

	@Override
	public void onCommandComplete(AsReader reader, CommandType command) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onCommandComplete(%s)", command);
	}

	@Override
	public void onReadBarcode(AsReader reader, BarcodeType type, String codeId, String barcode) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadBarcode (%s, %s, [%s])", type, codeId, barcode);
	}

	@Override
	public void onExceptionOccurred(AsReader reader, int code) {
		ExceptionCode exceptCode = ExceptionCode.valueOf(code);
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onExceptionOccurred [%s]", exceptCode);
	}

	@Override
	public void onKeyEvent(AsReader reader, KeyType key, KeyState state) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onKeyEvent [%s, %s]", key, state);
	}

	// Initialize Widgets
	private void initWidgets() {

		txtDemoVersion = (TextView) findViewById(R.id.demo_version);
		txtSDKVersion = (TextView) findViewById(R.id.sdk_version);
		txtFirmwareVersion = (TextView) findViewById(R.id.firmware_version);
		txtAddress = (TextView) findViewById(R.id.address);
		txtGlobalBand = (TextView) findViewById(R.id.global_band);
		txtSerialNumber = (TextView) findViewById(R.id.serial_number);
		btnInventory = (Button) findViewById(R.id.inventory);
		btnInventory.setOnClickListener(this);
		btnInventory.setOnTouchListener(this);
		btnReadMemory = (Button) findViewById(R.id.read_memory);
		btnReadMemory.setOnClickListener(this);
		btnReadMemory.setOnTouchListener(this);
		btnWriteMemory = (Button) findViewById(R.id.write_memory);
		btnWriteMemory.setOnClickListener(this);
		btnWriteMemory.setOnTouchListener(this);
		btnTagAccess = (Button) findViewById(R.id.tag_access);
		btnTagAccess.setOnClickListener(this);
		btnTagAccess.setOnTouchListener(this);
		btnOption = (Button) findViewById(R.id.option);
		btnOption.setOnClickListener(this);
		btnOption.setOnTouchListener(this);
		btnBarcodeDemo = (Button) findViewById(R.id.barcode_demo);
		btnBarcodeDemo.setOnClickListener(this);
		btnBarcodeDemo.setOnTouchListener(this);
		btnBarcodeOption = (Button) findViewById(R.id.barcode_option);
		btnBarcodeOption.setOnClickListener(this);
		btnBarcodeOption.setOnTouchListener(this);
		imgBattery = (ImageView) findViewById(R.id.battery);
		imgLogo = (ImageView) findViewById(R.id.app_logo);
		imgLogo.setOnClickListener(this);
		btnLedTest = (Button) findViewById(R.id.led_test);
		btnLedTest.setOnClickListener(this);

		setButtonTextColor(COLOR_DISABLED);
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	// Display Application Version
	private void displayVersion() {
		String versionName = "";
		String packageName = getPackageName();

		// Get Application Version
		try {
			versionName = getPackageManager().getPackageInfo(packageName, PackageManager.GET_META_DATA).versionName;
		} catch (NameNotFoundException e) {
			versionName = "";
		}
		txtDemoVersion.setText(versionName);

		if (mReader!=null)
			txtSDKVersion.setText(mReader.getVersion());

		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayVersion() - [%s]", versionName);
	}

	// Setup Always Wakeup
	@SuppressWarnings("deprecation")
	private void setupAlwaysWakeup() {
		if (wakeLock != null)
			return;

		PowerManager powerManager = (PowerManager) this.getSystemService(Activity.POWER_SERVICE);
		if(powerManager != null) {
			wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK, APP_NAME);
			wakeLock.acquire(1000 );
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. setupAlwayWakeup()");
	}

	// Set Default Wakeup
	private void defaultWakeup() {
		if (wakeLock == null)
			return;

		if(wakeLock.isHeld())
			wakeLock.release();

		wakeLock = null;

		ASRLog.i(TAG, ASRLog.INFO, "INFO. defaultWakeup()");
	}

	// Initialize RFID Reader
	private void initAsReader() {
		mReader = AsReaderManager.getInstance();
		if(mReader == null) {
			Toast.makeText(this, "getInstance failed", Toast.LENGTH_SHORT).show();
			return;
		}

		mReader.setEventListener(this);
		mHandler = new Handler();
		mHandler.post(mConnectDevice);
	}
	
	private Runnable mConnectDevice = new Runnable() {
		@Override
		public void run() {
			if(mDevice == null) {
				mDevice = new DeviceUsbCdc(getApplicationContext());
			}
			mReader.connectDevice(mDevice);
			ASRLog.i(TAG, ASRLog.INFO, "INFO. initAsReader()");
		}
	};

	// Enable/Disable All Menu Button
	private void enableMenuButtons(boolean enabled) {
		btnInventory.setEnabled(enabled && mReader.isRfidEnabled());
		btnInventory.setTextColor(enabled  && mReader.isRfidEnabled() ? COLOR_ENABLED : COLOR_DISABLED);
		btnOption.setEnabled(enabled && mReader.isRfidEnabled());
		btnOption.setTextColor(enabled  && mReader.isRfidEnabled() ? COLOR_ENABLED : COLOR_DISABLED);
		btnReadMemory.setEnabled(enabled && mReader.isRfidEnabled());
		btnReadMemory.setTextColor(enabled  && mReader.isRfidEnabled() ? COLOR_ENABLED : COLOR_DISABLED);
		btnWriteMemory.setEnabled(enabled && mReader.isRfidEnabled());
		btnWriteMemory.setTextColor(enabled  && mReader.isRfidEnabled() ? COLOR_ENABLED : COLOR_DISABLED);
		btnTagAccess.setEnabled(enabled && mReader.isRfidEnabled());
		btnTagAccess.setTextColor(enabled  && mReader.isRfidEnabled() ? COLOR_ENABLED : COLOR_DISABLED);
		btnBarcodeDemo.setEnabled(enabled && mReader.isBarcodeEnabled());
		btnBarcodeDemo.setTextColor(enabled && mReader.isBarcodeEnabled() ? COLOR_ENABLED : COLOR_DISABLED);
		btnBarcodeOption.setEnabled(enabled && mReader.isBarcodeEnabled());
		btnBarcodeOption.setTextColor(enabled && mReader.isBarcodeEnabled() ? COLOR_ENABLED : COLOR_DISABLED);
		btnLedTest.setEnabled(enabled && mReader.isRfidEnabled());
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableMenuButtons(%s)", enabled);
	}

	// Visible Menu Item
	private void visibleMenuItem(ConnectionState state) {
		if(mnuConnectDevice != null)
			mnuConnectDevice.setVisible(state == ConnectionState.Disconnected);
		if(mnuDisconnectDevice != null)
			mnuDisconnectDevice.setVisible(state != ConnectionState.Disconnected);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. visibleMenuItem(%s)", state);
	}

	// Load Configuration
	private void loadConfig() {
		SharedPreferences prefs = getSharedPreferences(APP_NAME, MODE_PRIVATE);
		mBatteryInterval = prefs.getInt(KEY_BATTERY_INTERVAL, DEFAULT_BATTERY_INTERVAL);
		if (mBatteryInterval > 60)
			mBatteryInterval = DEFAULT_BATTERY_INTERVAL;
		ASRLog.d(TAG, ASRLog.INFO, "DEBUG. loadConfig() - Batery Check Interval : [%d]", mBatteryInterval);
		mMaskType = MaskType.valueOf(prefs.getInt(KEY_MASK_TYPE, DEFAULT_MASK_TYPE));
		ASRLog.i(TAG, ASRLog.INFO, "DEBUG. loadConfig() - Mask Type : [%s]", mMaskType);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. loadConfig()");
	}

	// Save Configuration
	private void saveConfig() {
		SharedPreferences prefs = getSharedPreferences(APP_NAME, MODE_PRIVATE);
		SharedPreferences.Editor editor = prefs.edit();

		editor.putInt(KEY_BATTERY_INTERVAL, mBatteryInterval);
		ASRLog.d(TAG, ASRLog.INFO, "DEBUG. saveConfig() - Batery Check Interval : [%d]", mBatteryInterval);
		editor.putInt(KEY_MASK_TYPE, mMaskType.getCode());
		ASRLog.d(TAG, ASRLog.INFO, "DEBUG. saveConfig() - Mask Type : [%s]", mMaskType);
		editor.apply();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. saveConfig()");
	}

	private void startCheckBattery() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. startCheckBattery()");

		if(mBatteryInterval == 0)
			return;

		if(timerCheckBattery != null)
			return;

		int interval = mBatteryInterval * 1000;
		timerCheckBattery = new Timer();
		timerCheckBattery.schedule(new TimerTask() {

			@Override
			public void run() {
				runOnUiThread(mActionCheckBattery);
			}

		}, 1000, interval);
	}

	private void stopCheckBattery() {
		if (timerCheckBattery == null)
			return;
		timerCheckBattery.cancel();
		timerCheckBattery = null;
		ASRLog.i(TAG, ASRLog.INFO, "INFO. stopCheckBattery()");
	}

	private Runnable mActionCheckBattery = new Runnable() {

		@Override
		public void run() {
			if (timerCheckBattery != null)
				checkBattery();
		}

	};

	private void checkBattery() {

		int battery = 0;

		try {
			battery = mReader.getBatteryStatus();
		} catch (AsReaderException e) {
			battery = 0;
		}

		switch (battery) {
		case 0:
			imgBattery.setImageResource(R.drawable.battery0);
			break;
		case 1:
			imgBattery.setImageResource(R.drawable.battery1);
			break;
		case 2:
			imgBattery.setImageResource(R.drawable.battery2);
			break;
		case 3:
			imgBattery.setImageResource(R.drawable.battery3);
			break;
		case 4:
			imgBattery.setImageResource(R.drawable.battery4);
			break;
		default:
			return;
		}
		ASRLog.i(TAG, ASRLog.INFO, "INFO. checkBattery() - [%d]", battery); 
	}

	private void setButtonTextColor(int color) {
		btnInventory.setTextColor(color);
		btnReadMemory.setTextColor(color);
		btnWriteMemory.setTextColor(color);
		btnTagAccess.setTextColor(color);
		btnOption.setTextColor(color);
		btnBarcodeDemo.setTextColor(color);
		btnBarcodeOption.setTextColor(color);
	}
	
	private void setDisconnectedState() {

		stopCheckBattery();

		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				txtFirmwareVersion.setText("");
				txtAddress.setText("");
				txtSerialNumber.setText("");
				txtGlobalBand.setText("");

				imgBattery.setImageResource(R.drawable.battery);
				imgLogo.setImageResource(R.drawable.asreader_log_off);
				enableMenuButtons(false);

				visibleMenuItem(ConnectionState.Disconnected);
				WaitDialog.hide();
			}
		});

		ASRLog.i(TAG, ASRLog.INFO, "INFO. setDisconnectedState()");
	}

	private void showDialog(String msg) {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.basic_alert);
		builder.setMessage(msg);
		builder.setCancelable(false);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				mIsAlert = true;
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showDialog().$PositiveButton.onClick()");
			}
		});

		AlertDialog dialog = builder.create();
		dialog.show();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. showDialog()");
	}

	private void showLEDControlDialog() {

		boolean left = mReader.getModeLeftLedOn();
		boolean right = mReader.getModeRightLedOn();

		LayoutInflater layoutInflater = LayoutInflater.from(this);
		View promptView = layoutInflater.inflate(R.layout.dialog_ledcontrol, null);
		AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(this);
		alertDialogBuilder.setView(promptView);

		final Switch switch_led_left      = (Switch) promptView.findViewById(R.id.led_left_swich);
		final Switch switch_led_right     = (Switch) promptView.findViewById(R.id.led_right_swich);

		switch_led_left.setChecked(left);
		switch_led_right.setChecked(right);

		switch_led_left.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton compoundButton, boolean isChecked) {
				if(isChecked)
					mReader.setModeLeftLedOn(true);
				else
					mReader.setModeLeftLedOn(false);
			}
		});
		switch_led_right.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton compoundButton, boolean isChecked) {
				if(isChecked)
					mReader.setModeRightLedOn(true);
				else
					mReader.setModeRightLedOn(false);
			}
		});

		// setup a dialog window
		alertDialogBuilder.setCancelable(false)
				.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int id) {
						dialog.cancel();
					}
				});

		// create an alert dialog
		AlertDialog alert = alertDialogBuilder.create();
		alert.show();
	}


	private String getRequestCode(int requestCode) {
		switch (requestCode) {
		case VIEW_INVENTORY:
			return "VIEW_INVENTORY";
		case AccessActivity.VIEW_READ_MEMORY:
			return "VIEW_READ_MEMORY";
		case AccessActivity.VIEW_WRITE_MEMORY:
			return "VIEW_WRITE_MEMORY";
		case AccessActivity.VIEW_TAG_ACCESS:
			return "VIEW_TAG_ACCESS";
		case VIEW_OPTION:
			return "VIEW_OPTION";
		case VIEW_BARCODE_DEMO:
			return "VIEW_BARCODE_DEMO";
		case VIEW_BARCODE_OPTION:
			return "VIEW_BARCODE_OPTION";
		}
		return "";
	}

	private String getResultCode(int resultCode) {
		switch (resultCode) {
		case Activity.RESULT_CANCELED:
			return "RESULT_CANCELED";
		case Activity.RESULT_OK:
			return "RESULT_OK";
		case Activity.RESULT_FIRST_USER:
			return "RESULT_FIRST_USER";
		case RESULT_DISCONNECTED:
			return "RESULT_DISCONNECTED";
		}
		return "";
	}

	private String getMenuId(int id) {
		switch(id) {
		case R.id.menu_connect_device:
			return "menu_connect_device";
		case R.id.menu_disconnect:
			return "menu_disconnect";
		}
		
		return "";
	}

	private String getOnClickId(int id) {
		switch (id) {
		case R.id.inventory:
			return "inventory";
		case R.id.read_memory:
			return "read_memory";
		case R.id.write_memory:
			return "write_memory";
		case R.id.tag_access:
			return "tag_access";
		case R.id.option:
			return "option";
		case R.id.barcode_demo:
			return "barcode_demo";
		case R.id.barcode_option:
			return "barcode_option";
		}
		return "";
	}

	@SuppressLint({ "NewApi", "ClickableViewAccessibility" })
	@Override
	public boolean onTouch(View v, MotionEvent event) {
	
		int id = v.getId();
		int action  = event.getAction();
		switch(id) {
			case R.id.inventory:
				switch(action) {
				case MotionEvent.ACTION_DOWN:
					btnInventory.setBackgroundColor(Color.LTGRAY);
					break;
				case MotionEvent.ACTION_UP:
				case MotionEvent.ACTION_CANCEL:
					btnInventory.setBackgroundColor(Color.TRANSPARENT);
					break;
				}
				break;
			case R.id.read_memory:
				switch(action) {
				case MotionEvent.ACTION_DOWN:
					btnReadMemory.setBackgroundColor(Color.LTGRAY);
					break;
				case MotionEvent.ACTION_UP:
				case MotionEvent.ACTION_CANCEL:
					btnReadMemory.setBackgroundColor(Color.TRANSPARENT);
					break;
				}
				break;
			case R.id.write_memory:
				switch(action) {
				case MotionEvent.ACTION_DOWN:
					btnWriteMemory.setBackgroundColor(Color.LTGRAY);
					break;
				case MotionEvent.ACTION_UP:
				case MotionEvent.ACTION_CANCEL:
					btnWriteMemory.setBackgroundColor(Color.TRANSPARENT);
					break;
				}
				break;
			case R.id.tag_access:
				switch(action) {
				case MotionEvent.ACTION_DOWN:
					btnTagAccess.setBackgroundColor(Color.LTGRAY);
					break;
				case MotionEvent.ACTION_UP:
				case MotionEvent.ACTION_CANCEL:
					btnTagAccess.setBackgroundColor(Color.TRANSPARENT);
					break;
				}
				break;
			case R.id.option:
				switch(action) {
				case MotionEvent.ACTION_DOWN:
					btnOption.setBackgroundColor(Color.LTGRAY);
					break;
				case MotionEvent.ACTION_UP:
				case MotionEvent.ACTION_CANCEL:
					btnOption.setBackgroundColor(Color.TRANSPARENT);
					break;
				}
				break;
			case R.id.barcode_demo:
				switch(action) {
				case MotionEvent.ACTION_DOWN:
					btnBarcodeDemo.setBackgroundColor(Color.LTGRAY);
					break;
				case MotionEvent.ACTION_UP:
				case MotionEvent.ACTION_CANCEL:
					btnBarcodeDemo.setBackgroundColor(Color.TRANSPARENT);
					break;
				}
				break;
			case R.id.barcode_option:
				switch(action) {
				case MotionEvent.ACTION_DOWN:
					btnBarcodeOption.setBackgroundColor(Color.LTGRAY);
					break;
				case MotionEvent.ACTION_UP:
				case MotionEvent.ACTION_CANCEL:
					btnBarcodeOption.setBackgroundColor(Color.TRANSPARENT);
					break;
				}
				break;
		}
		
		return false;
	}

	public class InitRunnable implements Runnable {

		private final String _firmwareVersion;
		private final String _serialNumber;
		private final String _rfVersion;
		private final String _globalBand;
		private final boolean _rfEnabled;

		public InitRunnable(String firmwareVersion, String serialNumber, String rfVersion, String globalBand, boolean rfEnabled) {
			_firmwareVersion = firmwareVersion;
			_serialNumber = serialNumber;
			_rfVersion = rfVersion;
			_globalBand = globalBand;
			_rfEnabled = rfEnabled;
		}

		@Override
		public void run() {
			txtFirmwareVersion.setText(_firmwareVersion);
			txtSerialNumber.setText(_serialNumber);
			if(_rfEnabled) {
				txtAddress.setText(_rfVersion);
				txtGlobalBand.setText(_globalBand);
			}

			startCheckBattery();
			WaitDialog.hide();
			visibleMenuItem(ConnectionState.Connected);
			imgLogo.setImageResource(R.drawable.asreader_log_on);
			enableMenuButtons(true);
			setProgressBarIndeterminateVisibility(false);
		}
	}

	
}
