package jp.co.asterisk.asreader.l251g.support.adapter;

import java.util.ArrayList;
import java.util.Locale;

import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.params.SelectMaskParam;
import com.asreader.lib.rfid.type.MaskActionType;
import com.asreader.lib.rfid.type.MaskTargetType;
import com.asreader.lib.rfid.type.MemoryBank;
import jp.co.asterisk.asreader.l251g.R;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

public class SelectMaskAdapter extends BaseAdapter {

	private static final String TAG = SelectMaskAdapter.class.getSimpleName();

	private LayoutInflater inflater;
	private ArrayList<SelectMaskItem> list;

	// ------------------------------------------------------------------------
	// Constructor
	// ------------------------------------------------------------------------
	public SelectMaskAdapter(Context context) {
		super();

		this.inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		this.list = new ArrayList<>();
	}

	public void addItemNoUpdate(SelectMaskParam param) {
		SelectMaskItem item = new SelectMaskItem();
		item.setTarget(param.getTarget());
		item.setAction(param.getAction());
		item.setBank(param.getBank());
		item.setOffset(param.getOffset());
		item.setMask(param.getMask());
		item.setLength(param.getLength());
		this.list.add(item);
		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. setItem([%s])", item.toString());
	}

	public void addItem(SelectMaskParam param) {
		addItemNoUpdate(param);
		this.notifyDataSetChanged();
	}

	public void updateItem(int index, SelectMaskParam param) {
		SelectMaskItem item = this.list.get(index);
		item.setTarget(param.getTarget());
		item.setAction(param.getAction());
		item.setBank(param.getBank());
		item.setOffset(param.getOffset());
		item.setMask(param.getMask());
		item.setLength(param.getLength());
		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. updateItem(%d, [%s])", index, item.toString());
		this.notifyDataSetChanged();
	}

	public void removeItem(int index) {
		this.list.remove(index);
		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. removeItem(%d)",index);
		this.notifyDataSetChanged();
	}
	
	public void clear() {
		this.list.clear();
		this.notifyDataSetChanged();
	}

	@Override
	public int getCount() {
		return this.list.size();
	}

	@Override
	public SelectMaskParam getItem(int position) {
		SelectMaskItem item;
		try {
			item = this.list.get(position);
		} catch (Exception e) {
			return null;
		}
		if (item == null) return null;

		return new SelectMaskParam(item.getTarget(),
				item.getAction(), item.getBank(), item.getOffset(),
				item.getMask(), item.getLength());
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		SelectMaskViewHolder holder;

		if (null == convertView) {
			convertView = this.inflater.inflate(R.layout.item_select_mask,
					parent, false);
			holder = new SelectMaskViewHolder(convertView, this);
		} else {
			holder = (SelectMaskViewHolder) convertView.getTag();
		}
		holder.setItem(position, this.list.get(position));
		return convertView;
	}

	// ------------------------------------------------------------------------
	// Internal SelectMaskItem Class
	// ------------------------------------------------------------------------
	private class SelectMaskItem {
		private MaskTargetType target;
		private MaskActionType action;
		private MemoryBank bank;
		private int offset;
		private String mask;
		private int length;

		private SelectMaskItem() {
			this.target = MaskTargetType.SL;
			this.action = MaskActionType.AB;
			this.bank = MemoryBank.EPC;
			this.offset = 16;
			this.mask = "";
			this.length = 0;
		}

		public MaskTargetType getTarget() {
			return this.target;
		}

		public void setTarget(MaskTargetType target) {
			this.target = target;
		}

		public MaskActionType getAction() {
			return this.action;
		}

		public void setAction(MaskActionType action) {
			this.action = action;
		}

		public MemoryBank getBank() {
			return this.bank;
		}

		public void setBank(MemoryBank bank) {
			this.bank = bank;
		}

		public int getOffset() {
			return this.offset;
		}

		public void setOffset(int offset) {
			this.offset = offset;
		}

		public String getMask() {
			return this.mask;
		}

		public void setMask(String mask) {
			this.mask = mask;
		}

		public int getLength() {
			return this.length;
		}

		public void setLength(int length) {
			this.length = length;
		}

		public String toString() {
			return "{" + this.target + ", " + this.action.toString(this.target)
					+ ", " + this.bank + ", " + this.offset + ", " + this.mask
					+ ", " + this.length + "}";
		}
	}

	// ------------------------------------------------------------------------
	// Internal SelectMaskViewHolder Class
	// ------------------------------------------------------------------------
	private class SelectMaskViewHolder {
		private TextView txtTarget;
		private TextView txtAction;
		private TextView txtBank;
		private TextView txtOffset;
		private TextView txtMask;
		private TextView txtLength;

		private SelectMaskViewHolder(View parent, SelectMaskAdapter adapter) {
			this.txtTarget = (TextView) parent.findViewById(R.id.target);
			this.txtAction = (TextView) parent.findViewById(R.id.action);
			this.txtBank = (TextView) parent.findViewById(R.id.bank);
			this.txtOffset = (TextView) parent.findViewById(R.id.offset);
			this.txtMask = (TextView) parent.findViewById(R.id.mask);
			this.txtLength = (TextView) parent.findViewById(R.id.length);
			parent.setTag(this);
		}

		public void setItem(int position, SelectMaskItem item) {
			this.txtTarget.setText(item.getTarget().toString());
			this.txtAction.setText(item.getAction().toString(item.getTarget()));
			this.txtBank.setText(item.getBank().toString());
			this.txtOffset.setText(String.format(Locale.US, "%d bit", item.getOffset()));
			this.txtMask.setText(item.getMask());
			this.txtLength.setText(String.format(Locale.US, "%d bit", item.getLength()));
		}
	}
}
