package jp.co.asterisk.asreader.l251g.support.view.base;

import android.content.Intent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;
import jp.co.asterisk.asreader.l251g.support.view.sub.EpcMaskActivity;
import jp.co.asterisk.asreader.l251g.support.view.sub.SelectMaskActivity;
import jp.co.asterisk.asreader.l251g.MainActivity;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.params.SelectMaskParam;
import com.asreader.lib.rfid.type.MaskActionType;
import com.asreader.lib.rfid.type.MaskTargetType;
import com.asreader.lib.rfid.type.MemoryBank;
import com.asreader.lib.rfid.type.QuerySL;
import com.asreader.lib.rfid.type.MaskType;
import jp.co.asterisk.asreader.l251g.R;

public abstract class RfidMaskActivity extends RfidPowerGainActivity implements OnClickListener {

	private static final String TAG = RfidMaskActivity.class.getSimpleName();

	public static final String KEY_MASK_TYPE = "mask_type";
	public static final String KEY_MASK = "mask_value";
	protected static final int DEFAULT_MASK_TYPE = 1;

	@SuppressWarnings("unused")
	protected static final String DEFAULT_MASK = "";

	private static final int EPC_OFFSET = 32;
	private static final int MAX_MASK = 8;

	protected Button btnMask;
	protected MaskType mMaskType;
	private String mMaskValue;

	private SelectMaskParam[] mBackupMasks;
	private QuerySL mBackupSL;

	public RfidMaskActivity() {
		mMaskValue = "";
	}

	@Override 
	public void onClick(View v) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		Intent intent;
		switch (v.getId()) {
		case R.id.mask:
			switch (mMaskType) {
				case Selection:
					intent = new Intent(this, SelectMaskActivity.class);
					break;
				case EPC:
					intent = new Intent(this, EpcMaskActivity.class);
					break;
				default:
					return;
			}
			startActivityForResult(intent, VIEW_MASK);
			break;
		}
		super.onClick(v);
	}

	@Override
	protected void initWidgets() {
		super.initWidgets();

		btnMask = (Button) findViewById(R.id.mask);
		btnMask.setOnClickListener(this);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		btnMask.setEnabled(isEnabledWidget(enabled) && mMaskValue.equals("") && mMaskType == MaskType.Selection);

		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	protected void initMask() {
		Intent intent = getIntent();
		int code = intent.getIntExtra(KEY_MASK_TYPE, DEFAULT_MASK_TYPE);

		mMaskType  = MainActivity.mUserConfigs.getMaskType();

		mMaskType = MaskType.valueOf(code);
		if ((mMaskValue = intent.getStringExtra(KEY_MASK)) == null)
			mMaskValue = "";

		if (mMaskValue.equals("")) {
			WaitDialog.hide();
			return;
		}

		if (mMaskType == MaskType.Selection) {
			try {
				mBackupSL = getReader().getQuerySL();
			} catch (AsReaderException e) {
				mBackupSL = QuerySL.NotUsed;
			}
			if (mBackupSL != QuerySL.NotUsed) {
				mBackupMasks = new SelectMaskParam[MAX_MASK];
				for (int i = 0; i < MAX_MASK; i++) {
					try {
						if (!getReader().usedSelectionMask(i))
							break;
						mBackupMasks[i] = getReader().getSelectionMask(i);
					} catch (AsReaderException e) {
						mBackupMasks[i] = null;
					}
				}
			}
		} else {
			mBackupMasks = null;
			mBackupSL = QuerySL.NotUsed;
		}
		
		try {
			getReader().clearSelectionMask();
		} catch (AsReaderException e1) {
			ASRLog.e(TAG, "ERROR. initMask() - Failed to clearSelectionMask");
		}

		try {
			getReader().setSelectionMask(0,
					new SelectMaskParam(MaskTargetType.SL, MaskActionType.AB, MemoryBank.EPC, EPC_OFFSET, mMaskValue));
			getReader().setQuerySL(QuerySL.SL);
		} catch (AsReaderException e) {
			ASRLog.e(TAG, e, "ERROR. initMask() - Failed to set selection mask [%s]", mMaskValue);
		}
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initMask() - [%s]", mMaskValue);
	}

	protected void exitMask() {

		if (mMaskValue.equals(""))
			return;

//		if (mMaskType != MaskType.Selection)
//			return;

		try {
			getReader().setQuerySL(mBackupSL);
		} catch (AsReaderException e) {
			ASRLog.e(TAG, e, "ERROR. exitMask() - Failed to rollback use selection mask [%s]", mBackupSL);
		}

		if (mBackupSL != QuerySL.NotUsed && mBackupMasks != null) {
			for (int i = 0; i < MAX_MASK; i++) {
				if (mBackupMasks[i] == null)
					break;
				try {
					getReader().setSelectionMask(i, mBackupMasks[i]);
				} catch (AsReaderException e) {
					ASRLog.e(TAG, e, "ERROR. exitMask() - Failed to rollback selection mask [%s]", mBackupMasks[i]);
				}
			}
		}
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. exitMask()");
	}

	protected MaskType getMaskType() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. getMaskType() - [%s]", mMaskType);
		return mMaskType;
	}

	protected String getMask() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. getMask() - [%s]", mMaskValue);
		return mMaskValue;
	}
}
