package jp.co.asterisk.asreader.l251g.support.view.sub;

import jp.co.asterisk.asreader.l251g.support.adapter.SelectMaskAdapter;
import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.IDialogResultListener;
import jp.co.asterisk.asreader.l251g.support.dialog.SelectMaskDialog;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import jp.co.asterisk.asreader.l251g.MainActivity;
import com.asreader.lib.device.type.ConnectionState;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.params.SelectMaskParam;
import com.asreader.lib.rfid.type.QuerySL;
import com.asreader.lib.rfid.type.QuerySession;
import com.asreader.lib.rfid.type.QueryTarget;
import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.AsReader;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Spinner;

import java.util.Locale;

public class SelectMaskActivity extends RfidActivity
		implements OnClickListener, IDialogResultListener, OnItemClickListener {

	private static final String TAG = SelectMaskActivity.class.getSimpleName();

	private static final int RESULT_DISCONNECTED = Activity.RESULT_FIRST_USER + 1;
	private static final int REMOVE_QUESTION_DIALOG = 2003;

	private static final int MIN_MASK = 0;
	private static final int MAX_MASK = 8;

	private ListView lstMasks;
	private Button btnAdd;
	private Button btnEdit;
	private Button btnRemove;
	private Spinner spnSelectFlag;
	private Spinner spnInventorySession;
	private Spinner spnSessionFlag;
	private Button btnSave;
	private Button btnCancel;

	private SelectMaskAdapter adpMasks;
	private SpinnerAdapter adpSelectFlag;
	private SpinnerAdapter adpInventorySession;
	private SpinnerAdapter adpSessionFlag;

	private SelectMaskDialog mDialog;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_selection_mask);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createReader();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initReader();

		// Enable action widget
		enableActionWidgets(false);

		// Load Selection Mask
		WaitDialog.show(this, R.string.load_selection_mask);

		new Thread(mLoadMask).start();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyReader();

		super.onDestroy();
	}

	@SuppressWarnings("deprecation")
	@Override
	public void onClick(View v) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.add_mask:
			showDialog(SelectMaskDialog.NEW_ITEM);
			break;
		case R.id.edit_mask:
			showDialog(SelectMaskDialog.MODIFY_ITEM);
			break;
		case R.id.remove_mask:
			showDialog(REMOVE_QUESTION_DIALOG);
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_selection_mask);
			new Thread(mSaveMask).start();
			break;
		case R.id.cancel:
			finish();
			break;
		}
	}

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
		switch (parent.getId()) {
		case R.id.mask_list:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onItemClick(%s, %d, %d)", ResUtil.getId(parent.getId()), position, id);
			enableActionWidgets(true);
			break;
		}
	}

	@Override
	protected Dialog onCreateDialog(int id) {
		switch (id) {
		case SelectMaskDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateDialog(NEW_ITEM)");
			if (mDialog == null) {
				mDialog = new SelectMaskDialog(this);
				mDialog.setResultListener(this);
			}
			return mDialog.getDialog();
		case SelectMaskDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateDialog(MODIFY_ITEM)");
			if (mDialog == null) {
				mDialog = new SelectMaskDialog(this);
				mDialog.setResultListener(this);
			}
			return mDialog.getDialog();
		case REMOVE_QUESTION_DIALOG:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateDialog(REMOVE_QUESTION_DIALOG)");
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setIcon(android.R.drawable.ic_dialog_alert);
			builder.setTitle(R.string.action_remove);
			builder.setMessage(R.string.remove_mask_message);
			builder.setPositiveButton(R.string.action_yes, new DialogInterface.OnClickListener() {

				@Override
				public void onClick(DialogInterface dialog, int which) {
					int position = lstMasks.getCheckedItemPosition();
					if (position < 0)
						return;
					lstMasks.setItemChecked(position, false);
					adpMasks.removeItem(position);
					enableActionWidgets(true);
				}
			});
			builder.setNegativeButton(R.string.action_no, null);
			return builder.create();
		}
		return null;
	}

	@Override
	protected void onPrepareDialog(int id, Dialog dialog) {
		switch (id) {
		case SelectMaskDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onPrepareDialog(NEW_ITEM)");
			mDialog.initDialog();
			break;
		case SelectMaskDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onPrepareDialog(MODIFY_ITEM)");
			int position = lstMasks.getCheckedItemPosition();
			SelectMaskParam param = adpMasks.getItem(position);
			mDialog.initDialog(param);
			break;
		}
	}

	@Override
	public void onOkClick(int what, DialogInterface dialog) {
		SelectMaskParam param;

		switch (what) {
		case SelectMaskDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onOkClick(NEW_ITEM)");
			param = mDialog.getItem();
			if(param != null) {
				adpMasks.addItem(param);
			}
			break;
		case SelectMaskDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onOkClick(MODIFY_ITEM)");
			int position = lstMasks.getCheckedItemPosition();
			param = mDialog.getItem();
			if(param != null) {
				adpMasks.updateItem(position, param);
			}
			break;
		}
		enableActionWidgets(true);
	}

	@Override
	public void onCancelClick(int what, DialogInterface dialog) {
		switch (what) {
		case SelectMaskDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCancelClick(NEW_ITEM)");
			break;
		case SelectMaskDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCancelClick(MODIFY_ITEM)");
			break;
		}
		enableActionWidgets(true);
	}

	@Override
	public void onStateChanged(AsReader reader, ConnectionState state) {

		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onStateChanged(%s)", state);
		if (state == ConnectionState.Disconnected) {
			setResult(RESULT_DISCONNECTED);
			finish();
		}
	}

	@Override
	protected void initReader() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initReader()");
	}

	// Initialize Widgets
	@Override
	protected void initWidgets() {
		lstMasks = (ListView) findViewById(R.id.mask_list);
		adpMasks = new SelectMaskAdapter(this);
		lstMasks.setAdapter(adpMasks);
		lstMasks.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		lstMasks.setOnItemClickListener(this);

		btnAdd = (Button) findViewById(R.id.add_mask);
		btnAdd.setOnClickListener(this);

		btnEdit = (Button) findViewById(R.id.edit_mask);
		btnEdit.setOnClickListener(this);

		btnRemove = (Button) findViewById(R.id.remove_mask);
		btnRemove.setOnClickListener(this);

		spnSelectFlag = (Spinner) findViewById(R.id.select_flag);
		adpSelectFlag = new SpinnerAdapter(this, android.R.layout.simple_list_item_1);
		spnSelectFlag.setAdapter(adpSelectFlag);

		spnInventorySession = (Spinner) findViewById(R.id.inventory_session);
		adpInventorySession = new SpinnerAdapter(this, android.R.layout.simple_list_item_1);
		spnInventorySession.setAdapter(adpInventorySession);

		spnSessionFlag = (Spinner) findViewById(R.id.session_flag);
		adpSessionFlag = new SpinnerAdapter(this, android.R.layout.simple_list_item_1);
		spnSessionFlag.setAdapter(adpSessionFlag);

		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);

		btnCancel = (Button) findViewById(R.id.cancel);
		btnCancel.setOnClickListener(this);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		int position = lstMasks.getCheckedItemPosition();
		int count = lstMasks.getCount();

		lstMasks.setEnabled(enabled);
		btnAdd.setEnabled(enabled && count < MAX_MASK);
		btnEdit.setEnabled(enabled && position >= 0);
		btnRemove.setEnabled(enabled && position >= 0);
		spnSelectFlag.setEnabled(enabled);
		spnInventorySession.setEnabled(enabled);
		spnSessionFlag.setEnabled(enabled);
		btnSave.setEnabled(enabled);
		btnCancel.setEnabled(enabled);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	// Asynchronous Load Selection Mask
	private Runnable mLoadMask = new Runnable() {

		@Override
		public void run() {

			ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. $mLoadMask.run()");

			SelectMaskParam param;

			try {
				//if (getReader().getQuerySL() == QuerySL.NotUsed) {
				if (MainActivity.mUserConfigs.getSL() == QuerySL.NotUsed) {
					ASRLog.d(TAG, ASRLog.DEBUG, "@@@ DEBUG. $mLoadMask.run() - Not used selection mask");
					runOnUiThread(mLoadedMask);
					return;
				}
			} catch (Exception e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadMask.run() - Failed to get used selection mask");
				runOnUiThread(mLoadedMask);
				return;
			}

			for (int i = AsReader.MIN_SELECTION_MASK; i < AsReader.MAX_SELECTION_MASK; i++) {
				try {
					//if (!getReader().usedSelectionMask(i))
					if (!MainActivity.mUserConfigs.usedSelectMask(i))
						break;
					//param = getReader().getSelectionMask(i);
					param = MainActivity.mUserConfigs.getSelectMask(i);
					ASRLog.d(TAG, ASRLog.DEBUG, "@@@ DEBUG. $mLoadMask.run() - Load Selection Mask [%d, [%s]]", i, param);
				} catch (Exception e) {
					ASRLog.e(TAG, e, "ERROR. $mLoadMask.run() - Failed to get selection mask [%d]", i);
					continue;
				}
				adpMasks.addItemNoUpdate(param);
			}
			runOnUiThread(mLoadedMask);

			ASRLog.i(TAG, ASRLog.INFO, "--- INFO. $mLoadMask.run()");
		}

	};

	private Runnable mLoadedMask = new Runnable() {

		@Override
		public void run() {
			ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. $mLoadedMask.run()");

			// Update Mask List
			adpMasks.notifyDataSetChanged();

			// Fill Select Flag Spinner
			for (QuerySL item : QuerySL.values()) {
				if (item == QuerySL.NotUsed)
					continue;
				adpSelectFlag.addItem(item.getValue(), item.toString());
			}
			adpSelectFlag.notifyDataSetChanged();

			// Fill Session Type Spinner
			for (QuerySession item : QuerySession.values()) {
				adpInventorySession.addItem(item.getValue(), item.toString());
			}
			adpInventorySession.notifyDataSetChanged();

			// Fill Flip Mode Spinner
			for (QueryTarget item : QueryTarget.values()) {
				adpSessionFlag.addItem(item.getValue(), item.toString());
			}
			adpSessionFlag.notifyDataSetChanged();

			// Update Select Flag
			try {
				//QuerySL sl = getReader().getQuerySL();
				QuerySL sl = MainActivity.mUserConfigs.getSL();
				if(sl == QuerySL.NotUsed)
					sl = QuerySL.All;
				spnSelectFlag.setSelection(adpSelectFlag.indexOf(sl.getValue()));
			} catch (Exception e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadedMask.run() - Failed to get select flag");
			}
			// Update Session Type
			try {
//				QuerySession sessionType = getReader().getQuerySession();
				QuerySession sessionType = MainActivity.mUserConfigs.getSession();
				spnInventorySession.setSelection(adpInventorySession.indexOf(sessionType.getValue()));
			} catch (Exception e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadedMask.run() - Failed to get inventory session");
			}
			// Update Session Flag
			try {
//				QueryTarget target = getReader().getQueryTarget();
				QueryTarget target = MainActivity.mUserConfigs.getTarget();
				spnSessionFlag.setSelection(adpSessionFlag.indexOf(target.getValue()));
			} catch (Exception e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadedMask.run() - Failed to get session flag");
			}

			enableActionWidgets(true);
			WaitDialog.hide();

			ASRLog.i(TAG, ASRLog.INFO, "--- INFO. $mLoadedMask.run()");
		}

	};

	// Asynchronous Save Selection Mask
	private Runnable mSaveMask = new Runnable() {

		@Override
		public void run() {
			ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. $mSaveMask.run()");

			SelectMaskParam param;
			for (int i = MIN_MASK; i < MAX_MASK; i++) {
				param = adpMasks.getItem(i);
				try {
					if (param == null) {
						getReader().removeSelectionMask(i);
						MainActivity.mUserConfigs.removeSelectMask(i);
					} else {
						getReader().setSelectionMask(i, param);
						MainActivity.mUserConfigs.setSelectMask(i, param);
						ASRLog.e(TAG, String.format(Locale.US, "%02d - %s", i, param));
					}
				} catch (AsReaderException e) {
					ASRLog.e(TAG, e, "ERROR. $mSaveMask.run() - Failed to set selection mask [%d, [%s]]", i, param);
					//continue;
				}
			}
			if (adpMasks.getCount() < 1) {
				try {
					getReader().clearSelectionMask();
					MainActivity.mUserConfigs.clearSelectMask();
				} catch (AsReaderException e) {
					ASRLog.e(TAG, e, "ERROR. $mSaveMask.run() - Failed to clear selection mask");
				}
			}else {
				QuerySL sl = QuerySL.valueOf(adpSelectFlag.getValue(spnSelectFlag.getSelectedItemPosition()));
				try {
					getReader().setQuerySL(sl);
					MainActivity.mUserConfigs.setSL(sl);
					ASRLog.e(TAG, String.format(Locale.US, "SL : %s", sl));
				} catch (AsReaderException e) {
					ASRLog.e(TAG, e, "ERROR. $mSaveMask.run() - Failed to set use selection mask [%s]", sl);
				}
			}

			QuerySession sessionType = QuerySession
					.valueOf(adpInventorySession.getValue(spnInventorySession.getSelectedItemPosition()));
			try {
				getReader().setQuerySession(sessionType);
				MainActivity.mUserConfigs.setSession(sessionType);
				ASRLog.e(TAG, String.format(Locale.US, "Session : %s", sessionType));
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveMask.run() - Failed to set inventory session [%s]", sessionType);
			}
			QueryTarget target = QueryTarget.valueOf(adpSessionFlag.getValue(spnSessionFlag.getSelectedItemPosition()));
			try {
				getReader().setQueryTarget(target);
				MainActivity.mUserConfigs.setTarget(target);
				ASRLog.e(TAG, String.format(Locale.US, "Target : %s", target));
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveMask.run() - Failed to set session flag [%s]", target);
			}

			MainActivity.mUserConfigs.Save();
			runOnUiThread(mSavedMask);

			ASRLog.i(TAG, ASRLog.INFO, "--- INFO. $mSaveMask.run()");
		}

	};

	private Runnable mSavedMask = new Runnable() {

		@Override
		public void run() {
			ASRLog.i(TAG, ASRLog.INFO, "INFO. $mSavedMask.run()");
			WaitDialog.hide();
			finish();
		}

	};

	
}
