package jp.co.asterisk.asreader.l251g.support.view.sub.barcode.symbol;

import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.filter.InputFilterMinMax;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.ParamValueList;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputFilter;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public class Code39SymbolActivity extends RfidActivity  implements OnClickListener {

	private static final String TAG = Code39SymbolActivity.class.getSimpleName();
	
	private static final int LENGTH_MIN = 0;
	private static final int LENGTH_MAX = 48;
	
	private TextView txtStartStopChar;
	private TextView txtCheckChar;
	private TextView txtMinLen;
	private TextView txtMaxLen;
	private TextView txtAppend;
	private TextView txtFullAscii;
	private Button btnSave;
	private Button btnDefault;
	
	private int mStartStopChar;
	private int mCheckChar;
	private int mMinLen;
	private int mMaxLen;
	private int mAppend;
	private int mFullAscii;
	
	private String[] mStartStopChars = new String[]{"Don't Transmit", "Transmit"};
	private String[] mCheckChars = new String[]{"No Check Character", "Validate, But Don't Transmit", "Validate, and Transmit"};
	private String[] mOnOffs = new String[]{"Off", "On"};
	
	private Handler mEvent;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_symbol_code39);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createReader();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initReader();

		// Enable action widget
		enableActionWidgets(true);
		
		mEvent = new Handler();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyReader();

		super.onDestroy();
	}
	
	

	@Override
	protected void onStart() {
		super.onStart();
		
		new Thread(mLoadOption).start();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	protected void initWidgets() {
		txtStartStopChar = (TextView) findViewById(R.id.code39_start_stop_char);
		txtStartStopChar.setOnClickListener(this);
		txtCheckChar = (TextView) findViewById(R.id.code39_check_char);
		txtCheckChar.setOnClickListener(this);
		txtMinLen = (TextView) findViewById(R.id.code39_length_min);
		txtMinLen.setOnClickListener(this);
		txtMaxLen = (TextView) findViewById(R.id.code39_length_max);
		txtMaxLen.setOnClickListener(this);
		txtAppend = (TextView) findViewById(R.id.code39_append);
		txtAppend.setOnClickListener(this);
		txtFullAscii = (TextView) findViewById(R.id.code39_full_ascii);
		txtFullAscii.setOnClickListener(this);
		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnDefault = (Button) findViewById(R.id.option_default);
		btnDefault.setOnClickListener(this);
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		btnSave.setEnabled(enabled);
		btnDefault.setEnabled(enabled);
	}

	@Override
	protected void initReader() {

	}
	
	private void displayValues() {
		
		txtStartStopChar.setText(mStartStopChars[mStartStopChar]);
		txtCheckChar.setText(mCheckChars[mCheckChar]);
		txtMinLen.setText(String.format(Locale.US, "%d", mMinLen));
		txtMaxLen.setText(String.format(Locale.US, "%d", mMaxLen));
		txtAppend.setText(mOnOffs[mAppend]);
		txtFullAscii.setText(mOnOffs[mFullAscii]);
	}
	
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = getReader().getBarcodeParam(new ParamName[] { ParamName.Code39StartStopChar, ParamName.Code39CheckChar,
															ParamName.Code39LengthMin, ParamName.Code39LengthMax, ParamName.Code39Append,
															ParamName.Code39FullAscii
			});
			
			mStartStopChar = values.getValue(ParamName.Code39StartStopChar);
			mCheckChar = values.getValue(ParamName.Code39CheckChar);
			mMinLen = values.getValue(ParamName.Code39LengthMin);
			mMaxLen = values.getValue(ParamName.Code39LengthMax);
			mAppend = values.getValue(ParamName.Code39Append);
			mFullAscii = values.getValue(ParamName.Code39FullAscii);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					displayValues();
					enableActionWidgets(true);
				}
				
			});
		}
		
	};
	
	private Runnable mDefaultOption = new Runnable() {

		@Override
		public void run() {
			getReader().setBarcodeParam(new ParamValue(ParamName.Code39Default));
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = new ParamValueList();
			values.add(ParamName.Code39StartStopChar, mStartStopChar);
			values.add(ParamName.Code39CheckChar, mCheckChar);
			values.add(ParamName.Code39LengthMin, mMinLen);
			values.add(ParamName.Code39LengthMax, mMaxLen);
			values.add(ParamName.Code39Append, mAppend);
			values.add(ParamName.Code39FullAscii, mFullAscii);
			
			getReader().setBarcodeParam(values);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private void showStartStopCharDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mStartStopChars.length; i++) {
			adapter.addItem(i, mStartStopChars[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.code39_start_stop_char);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mStartStopChar = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mStartStopChar);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showCheckCharDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mCheckChars.length; i++) {
			adapter.addItem(i, mCheckChars[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.code39_check_char);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mCheckChar = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mCheckChar);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showMinLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_min);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMinLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMinLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}
	
	private void showMaxLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_max);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMaxLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMaxLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}
	
	private void showCode39AppendDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mOnOffs.length; i++) {
			adapter.addItem(i, mOnOffs[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.code39_append);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mAppend = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mAppend);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showFullAsciiDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mOnOffs.length; i++) {
			adapter.addItem(i, mOnOffs[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.code39_full_ascii);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mFullAscii = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mFullAscii);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}

	@Override
	public void onClick(View v) {
		
		switch(v.getId()) {
		case R.id.code39_start_stop_char:
			showStartStopCharDialog();
			break;
		case R.id.code39_check_char:
			showCheckCharDialog();
			break;
		case R.id.code39_length_min:
			showMinLengthDialog();
			break;
		case R.id.code39_length_max:
			showMaxLengthDialog();
			break;
		case R.id.code39_append:
			showCode39AppendDialog();
			break;
		case R.id.code39_full_ascii:
			showFullAsciiDialog();
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_default:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.load_default_option);
			new Thread(mDefaultOption).start();
			break;
		}
	}

}
