package jp.co.asterisk.asreader.l251g.support.view.sub.barcode.symbol;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.ParamValueList;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputFilter;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.filter.InputFilterMinMax;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

public class Code93SymbolActivity extends RfidActivity implements OnClickListener{
	
	private static final String TAG = Code93SymbolActivity.class.getSimpleName();
	
	private ParamName mSymbolName;
	private TextView txtMinLen;
	private TextView txtMaxLen;
	private TextView lblAppend;
	private TextView txtAppend;
	private Button btnSave;
	private Button btnDefault;
	private Handler mEvent;
	
	private int mAppend;
	private int mMinLen;
	private int mMaxLen;
	
	private ParamName paramMinLength;
	private ParamName paramMaxLength;
	private ParamName paramAppend;
	private ParamName paramDefault;
	private int LENGTH_MIN = 0;

	@SuppressWarnings("unused")
	private int LENGTH_MAX;
	
	private String[] mOnOffs = new String[]{"Off", "On"};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_symbol_code93);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}
		
		this.mSymbolName = (ParamName) getIntent().getSerializableExtra("Symbol_Name");
		
		// Create Reader
		this.createReader();

		// Initialize Widget
		this.initWidgets();

		// Initialize Reader
		this.initReader();

		// Enable action widget
		this.enableActionWidgets(false);
		
		mEvent = new Handler();
		
	}
	
	@Override
	protected void onStart() {
		super.onStart();
		WaitDialog.show(this, R.string.barcode_load_param);
		new Thread(mLoadOption).start();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	protected void onDestroy() {
		// Destroy Reader
		this.destroyReader();
		
		super.onDestroy();
	}

	@Override
	protected void initReader() {
	}

	@Override
	protected void initWidgets() {
		txtMinLen = (TextView) findViewById(R.id.code93_length_min);
		txtMinLen.setOnClickListener(this);
		txtMaxLen = (TextView) findViewById(R.id.code93_length_max);
		txtMaxLen.setOnClickListener(this);
		lblAppend = (TextView) findViewById(R.id.label_append);
		txtAppend = (TextView) findViewById(R.id.code93_append);
		txtAppend.setOnClickListener(this);
		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnDefault = (Button) findViewById(R.id.option_default);
		btnDefault.setOnClickListener(this);
		
		if(mSymbolName == ParamName.QRCode) {
			this.setTitle(R.string.barcode_symbol_qr_code_name);
			this.lblAppend.setText(R.string.qrcode_append);
			this.paramMinLength = ParamName.QRCodeLengthMin;
			this.paramMaxLength = ParamName.QRCodeLengthMax;
			this.paramAppend = ParamName.QRCodeAppend;
			this.paramDefault = ParamName.QRCodeDefault;
			this.LENGTH_MIN = 7089;
		} else if(mSymbolName == ParamName.Matrix) {
			this.setTitle(R.string.barcode_symbol_datamatrix_name);
			this.lblAppend.setText(R.string.datamatrix_append);
			this.paramMinLength = ParamName.MatrixLengthMin;
			this.paramMaxLength = ParamName.MatrixLengthMax;
			this.paramAppend = ParamName.MatrixAppend;
			this.paramDefault = ParamName.MatrixDefault;
			this.LENGTH_MIN = 3116;
		} else if(mSymbolName == ParamName.AztecCode) {
			this.setTitle(R.string.barcode_symbol_aztec_name);
			this.lblAppend.setText(R.string.aztec_append);
			this.paramMinLength = ParamName.AztecCodeLengthMin;
			this.paramMaxLength = ParamName.AztecCodeLengthMax;
			this.paramAppend = ParamName.AztecAppend;
			this.paramDefault = ParamName.AztecCodeDefault;
			this.LENGTH_MIN = 3832;
		} else if(mSymbolName == ParamName.Code93) {
			this.paramMinLength = ParamName.Code93LengthMin;
			this.paramMaxLength = ParamName.Code93LengthMax;
			this.paramAppend = ParamName.Code93Append;
			this.paramDefault = ParamName.Code93Default;
			this.LENGTH_MIN = 80;
		}
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		btnSave.setEnabled(enabled);
		btnDefault.setEnabled(enabled);
	}

	private void displayValues() {
		txtMinLen.setText(String.format(Locale.US, "%d", mMinLen));
		txtMaxLen.setText(String.format(Locale.US, "%d", mMaxLen));
		txtAppend.setText(mOnOffs[mAppend]);
	}
	
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = getReader().getBarcodeParam(new ParamName[] { paramMinLength, paramMaxLength, paramAppend});
			
			mMinLen = values.getValue(paramMinLength);
			mMaxLen = values.getValue(paramMaxLength);
			mAppend = values.getValue(paramAppend);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					displayValues();
					enableActionWidgets(true);
					WaitDialog.hide();
				}
				
			});
		}
		
	};
	
	private Runnable mDefaultOption = new Runnable() {

		@Override
		public void run() {

			getReader().setBarcodeParam(new ParamValue(paramDefault));
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = new ParamValueList();
			
			values.add(paramMinLength, mMinLen);
			values.add(paramMaxLength, mMaxLen);
			values.add(paramAppend, mAppend);
			
			getReader().setBarcodeParam(values);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private void showMinLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_min);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMinLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMinLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}
	
	private void showMaxLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_max);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMaxLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMaxLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}
	
	private void showAppendDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mOnOffs.length; i++) {
			adapter.addItem(i, mOnOffs[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.code93_append);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mAppend = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mAppend);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}

	@Override
	public void onClick(View v) {
		switch(v.getId()) {
		case R.id.code93_length_min:
			showMinLengthDialog();
			break;
		case R.id.code93_length_max:
			showMaxLengthDialog();
			break;
		case R.id.code93_append:
			showAppendDialog();
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_default:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.load_default_option);
			new Thread(mDefaultOption).start();
			break;
		}
	}

}
