package jp.co.asterisk.asreader.l251g.support.view.sub.barcode.symbol;

import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.filter.InputFilterMinMax;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.ParamValueList;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputFilter;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public class I2of5SymbolActivity extends RfidActivity implements OnClickListener {

	private static final String TAG = I2of5SymbolActivity.class.getSimpleName();
	
	private static final int LENGTH_MIN = 0;
	private static final int LENGTH_MAX = 80;
	
	private TextView txtCheckDigit;
	private TextView txtMinLen;
	private TextView txtMaxLen;
	private Button btnSave;
	private Button btnDefault;
	
	ParamName paramMinLength;
	ParamName paramMaxLength;
	ParamName paramCheckDigit;
	ParamName paramDefault;
	
	private int mCheckDigit;
	private int mMinLen;
	private int mMaxLen;
	
	private String[] mCheckDigits = new String[]{"No Check Character", "Validate, But Don't Transmit", "Validate, and Transmit"};
	
	private ParamName mSymbolName;
	
	private Handler mEvent;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_symbol_i2of5);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");
		
		this.mSymbolName = (ParamName) getIntent().getSerializableExtra("Symbol_Name");

		// Create Reader
		this.createReader();

		// Initialize Widget
		this.initWidgets();

		// Initialize Reader
		this.initReader();

		// Enable action widget
		this.enableActionWidgets(true);
		
		mEvent = new Handler();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		this.destroyReader();

		super.onDestroy();
	}
	
	

	@Override
	protected void onStart() {
		super.onStart();
		new Thread(mLoadOption).start();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	protected void initWidgets() {
		txtCheckDigit = (TextView) findViewById(R.id.i2of5_check_digit);
		txtCheckDigit.setOnClickListener(this);
		txtMinLen = (TextView) findViewById(R.id.i2of5_length_min);
		txtMinLen.setOnClickListener(this);
		txtMaxLen = (TextView) findViewById(R.id.i2of5_length_max);
		txtMaxLen.setOnClickListener(this);
		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnDefault = (Button) findViewById(R.id.option_default);
		btnDefault.setOnClickListener(this);
		
		if(mSymbolName == ParamName.I2of5) {
			this.setTitle(R.string.barcode_symbol_interleaved2of5_name);
			paramMinLength = ParamName.I2of5LengthMin;
			paramMaxLength = ParamName.I2of5LengthMax;
			paramCheckDigit = ParamName.I2of5CheckDigit;
			paramDefault = ParamName.I2of5Default;
		} else {
			this.setTitle(R.string.barcode_symbol_nec2of5_name);
			paramMinLength = ParamName.NEC2of5LengthMin;
			paramMaxLength = ParamName.NEC2of5LengthMax;
			paramCheckDigit = ParamName.NEC2of5CheckDigit;
			paramDefault = ParamName.NEC2of5Default;
		}
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		btnSave.setEnabled(enabled);
		btnDefault.setEnabled(enabled);
	}

	@Override
	protected void initReader() {

	}
	
	private void displayValues() {
		txtCheckDigit.setText(mCheckDigits[mCheckDigit]);
		txtMinLen.setText(String.format(Locale.US, "%d", mMinLen));
		txtMaxLen.setText(String.format(Locale.US, "%d", mMaxLen));
	}
	
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
		ParamValueList values = getReader().getBarcodeParam(new ParamName[] { paramMinLength, paramMaxLength, paramCheckDigit});

		values.getValue(paramMinLength);
		values.getValue(paramMaxLength);
		values.getValue(paramCheckDigit);

		mEvent.post(new Runnable() {

			@Override
			public void run() {
				displayValues();
				enableActionWidgets(true);
			}

		});
		}
		
	};
	
	private Runnable mDefaultOption = new Runnable() {

		@Override
		public void run() {
		getReader().setBarcodeParam(new ParamValue(paramDefault));

		mEvent.post(new Runnable() {

			@Override
			public void run() {
				WaitDialog.hide();
				finish();
			}

		});
		}
		
	};
	
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			
		ParamValueList values = new ParamValueList();

		values.add(paramMinLength, mMinLen);
		values.add(paramMaxLength, mMaxLen);
		values.add(paramCheckDigit, mCheckDigit);

		getReader().setBarcodeParam(values);

		mEvent.post(new Runnable() {

			@Override
			public void run() {
				WaitDialog.hide();
				finish();
			}

		});
		}
		
	};
	
	private void showCheckDigitDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mCheckDigits.length; i++) {
			adapter.addItem(i, mCheckDigits[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		
		builder.setTitle(R.string.i2of5_check_digit);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mCheckDigit = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mCheckDigit);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showMinLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_min);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMinLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMinLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}
	
	private void showMaxLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_max);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMaxLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMaxLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}

	@Override
	public void onClick(View v) {
		switch(v.getId()) {
		case R.id.i2of5_check_digit:
			showCheckDigitDialog();
			break;
		case R.id.i2of5_length_min:
			showMinLengthDialog();
			break;
		case R.id.i2of5_length_max:
			showMaxLengthDialog();
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_default:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.load_default_option);
			new Thread(mDefaultOption).start();
			break;
		}
	}

}
