package jp.co.asterisk.asreader.l251g.support.view.sub.barcode.symbol;

import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValueList;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;

import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public class PostalSymbolActivity extends RfidActivity implements OnClickListener {

	private static final String TAG = PostalSymbolActivity.class.getSimpleName();
	
	private TextView txt2dPostalCode;
	private TextView txtPlanetCoceCheckDigit;
	private TextView txtPostnetCheckDigit;
	private TextView txtAusPostInterpretation;
	private Button btnSave;
	private Button btnDefault;
	private Handler mEvent;
	
	private int m2dPostalCode;
	private int mPlanetCoceCheckDigit;
	private int mPostnetCheckDigit;
	private int mAusPostInterpretation;
	
	private String[] m2DPostalCodes = new String[]{"Off", "Australian Post On", "InfoMail On", "Japanese Post On", "KIX Post On",
												"Planet Code On", "Postnet On", "British Post On", "InfoMail and British Post On",
												"Postal-4i On", "Intelligent Mail Bar Code On", "Postnet with B and B’ Fields On",
												"Planet and Postnet On", "Planet and Postal-4i On", "Postnet and Postal-4i On",
												"Planet and Intelligent Mail Bar Code On", "Postnet and Intelligent Mail Bar Code On",
												"Postal-4i and Intelligent Mail Bar Code On", "Planet and Postnet with B and B’ Fields On",
												"Postal-4i and Postnet with B and B’ Fields On", 
												"Intelligent Mail Bar Code and Postnet with B and B’ Fields On",
												"Planet, Postnet, and Postal-4i On", "Planet, Postnet, and Intelligent Mail Bar Code On",
												"Planet, Postal-4i, and Intelligent Mail Bar Code On",
												"Postnet, Postal-4i, and Intelligent Mail Bar Code On",
												"Planet, Postal-4i, and Postnet with B and B’ Fields On",
												"Planet, Intelligent Mail Bar Code, and Postnet with B and B’ Fields On",
												"Postal-4i, Intelligent Mail Bar Code, and Postnet with B and B’ Fields On",
												"Planet, Postal-4i, Intelligent Mail Bar Code, and Postnet On",
												"Planet, Postal-4i, Intelligent Mail Bar Code, and Postnet with B and B’ Fields On",
												"Canadian Post On"};
	private String[] mCheckDigits = new String[]{"Don’t Transmit", "Transmit"};
	private String[] mAustralianPostInterpretations = new String[]{"Bar Output", "Numeric N Table", "Alphanumeric C Table",
																	"Combination N and C Tables"};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_symbol_postal);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		this.createReader();

		// Initialize Widget
		this.initWidgets();

		// Initialize Reader
		this.initReader();

		// Enable action widget
		this.enableActionWidgets(true);
		
		mEvent = new Handler();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		this.destroyReader();

		super.onDestroy();
	}
	
	

	@Override
	protected void onStart() {
		super.onStart();
		
		WaitDialog.show(this, R.string.barcode_load_param);
		new Thread(mLoadOption).start();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	protected void initWidgets() {
		txt2dPostalCode = (TextView) findViewById(R.id.postal_codes_2d);
		txt2dPostalCode.setOnClickListener(this);
		txtPlanetCoceCheckDigit = (TextView) findViewById(R.id.planet_code_check_digit);
		txtPlanetCoceCheckDigit.setOnClickListener(this);
		txtPostnetCheckDigit = (TextView) findViewById(R.id.postnet_check_digit);
		txtPostnetCheckDigit.setOnClickListener(this);
		txtAusPostInterpretation = (TextView) findViewById(R.id.aus_post_interpretation);
		txtAusPostInterpretation.setOnClickListener(this);
		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnDefault = (Button) findViewById(R.id.option_default);
		btnDefault.setOnClickListener(this);
		btnDefault.setEnabled(false);
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		btnSave.setEnabled(enabled);
		//btnDefault.setEnabled(enabled);
	}

	@Override
	protected void initReader() {

	}
	
	private void displayValues() {
		txt2dPostalCode.setText(m2DPostalCodes[m2dPostalCode]);
		txtPlanetCoceCheckDigit.setText(mCheckDigits[mPlanetCoceCheckDigit]);
		txtPostnetCheckDigit.setText(mCheckDigits[mPostnetCheckDigit]);
		txtAusPostInterpretation.setText(mAustralianPostInterpretations[mAusPostInterpretation]);
	}
	
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = getReader().getBarcodeParam(new ParamName[] { ParamName.PostalCodes, ParamName.PlanetCodeCheckDigit,
					ParamName.PostnetCheckDigit, ParamName.AustralianPostInterpretation});
			
			m2dPostalCode = values.getValue(ParamName.PostalCodes);
			mPlanetCoceCheckDigit = values.getValue(ParamName.PlanetCodeCheckDigit);
			mPostnetCheckDigit = values.getValue(ParamName.PostnetCheckDigit);
			mAusPostInterpretation = values.getValue(ParamName.AustralianPostInterpretation);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					displayValues();
					enableActionWidgets(true);
					WaitDialog.hide();
				}
				
			});
		}
		
	};
	
	private Runnable mDefaultOption = new Runnable() {

		@Override
		public void run() {

			// Not supported.
			//getReader().setBarcodeParam(new ParamValue(ParamName.));
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = new ParamValueList();
			
			values.add(ParamName.PostalCodes, m2dPostalCode);
			values.add(ParamName.PlanetCodeCheckDigit, mPlanetCoceCheckDigit);
			values.add(ParamName.PostnetCheckDigit, mPostnetCheckDigit);
			values.add(ParamName.AustralianPostInterpretation, mAusPostInterpretation);
			
			getReader().setBarcodeParam(values);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private void show2DPostalCodeDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<m2DPostalCodes.length; i++) {
			adapter.addItem(i, m2DPostalCodes[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.postal_codes_2d);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					m2dPostalCode = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(m2dPostalCode);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showPlanetCodeCheckDigitDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mCheckDigits.length; i++) {
			adapter.addItem(i, mCheckDigits[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.planet_code_check_digit);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mPlanetCoceCheckDigit = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mPlanetCoceCheckDigit);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showPostnetCheckDigitDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mCheckDigits.length; i++) {
			adapter.addItem(i, mCheckDigits[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.postnet_check_digit);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mPostnetCheckDigit = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mPostnetCheckDigit);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showAusPostInterpretationDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mAustralianPostInterpretations.length; i++) {
			adapter.addItem(i, mAustralianPostInterpretations[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.aus_post_interpretation);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mAusPostInterpretation = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mAusPostInterpretation);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}

	@Override
	public void onClick(View v) {
		switch(v.getId()) {
		case R.id.postal_codes_2d:
			show2DPostalCodeDialog();
			break;
		case R.id.planet_code_check_digit:
			showPlanetCodeCheckDigitDialog();
			break;
		case R.id.postnet_check_digit:
			showPostnetCheckDigitDialog();
			break;
		case R.id.aus_post_interpretation:
			showAusPostInterpretationDialog();
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_default:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.load_default_option);
			new Thread(mDefaultOption).start();
			break;
		}
	}

}
