package jp.co.asterisk.asreader.l251g.view;

import jp.co.asterisk.asreader.l251g.support.adapter.BarcodeSymbolListAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;
import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

public class BarcodeOptionActivity extends RfidActivity implements OnClickListener, OnItemClickListener {

	private static final String TAG = BarcodeOptionActivity.class.getSimpleName();

	private ListView lstSymbols;
	private Button btnDefault;
	private Button btnDisableAll;
	private Button btnEnableAll;
	private Button btnSave;
	private BarcodeSymbolListAdapter adpSymbols;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}
		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.barcode_option_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		this.createReader();

		// Initialize Widget
		this.initWidgets();

		// Initialize Reader
		this.initReader();

		// Enable action widget
		this.enableActionWidgets(false);

		WaitDialog.show(this, R.string.load_option);
		new Thread(mLoadSymbol).start();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");
		// Destroy Reader
		destroyReader();

		super.onDestroy();
	}

	@Override
	protected void onResume() {
		super.onResume();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onResume()");
	}

	@Override
	protected void onPause() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onPause()");
		super.onPause();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));
		
		switch(requestCode) {
		case VIEW_BARCODE_OPTION_GS1_COMPOSITE:
			if(resultCode == Activity.RESULT_OK) {
				adpSymbols.setItem(ParamName.GS1Emulation, data.getBooleanExtra("GS1Emulation", false));
				adpSymbols.refreshAdapter();
				enableActionWidgets(true);
			} else {
				enableActionWidgets(true);
			}
			break;
		default:
			enableActionWidgets(true);
			break;
		}
	}

	@Override
	public void onClick(View v) {

		switch (v.getId()) {
		case R.id.default_all_symbologies:
			adpSymbols.defaultAll();
			break;
		case R.id.disable_all_symbologies:
			adpSymbols.disableAll();
			break;
		case R.id.enable_all_symbologies:
			adpSymbols.enableAll();
			break;
		case R.id.save_symbologies:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.baroce_save_symbols);
			new Thread(mSaveSymbol).start();
			break;
		}
	}

	@Override
	protected void initWidgets() {
		lstSymbols = (ListView) findViewById(R.id.symbol_list);
		lstSymbols.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE);
		adpSymbols = new BarcodeSymbolListAdapter(this);
		lstSymbols.setAdapter(adpSymbols);
		lstSymbols.setOnItemClickListener(this);
		btnDefault = (Button) findViewById(R.id.default_all_symbologies);
		btnDefault.setOnClickListener(this);
		btnDisableAll = (Button) findViewById(R.id.disable_all_symbologies);
		btnDisableAll.setOnClickListener(this);
		btnEnableAll = (Button) findViewById(R.id.enable_all_symbologies);
		btnEnableAll.setOnClickListener(this);
		btnSave = (Button) findViewById(R.id.save_symbologies);
		btnSave.setOnClickListener(this);
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		lstSymbols.setEnabled(enabled);
		btnDefault.setEnabled(enabled);
		btnDisableAll.setEnabled(enabled);
		btnEnableAll.setEnabled(enabled);
		btnSave.setEnabled(enabled);
	}

	@Override
	protected void initReader() {
	}

	private Runnable mLoadSymbol = new Runnable() {

		@Override
		public void run() {
			ParamValue value;

			ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. Load Symbologies");

			lstSymbols.setAdapter(null);

			try {
				ParamName[] names = new ParamName[] {
						ParamName.Codabar,
						ParamName.Code39,
						ParamName.Code32Pharmaceutical,
						ParamName.I2of5,
						ParamName.NEC2of5,
						ParamName.Code93,
						ParamName.R2of5,
						ParamName.A2of5,
						ParamName.X2of5,
						ParamName.Code11,
						ParamName.Code128,
						ParamName.GS1128,
						ParamName.Telepen,
						ParamName.UPCA,
						ParamName.UPCACouponCode,
						ParamName.CouponGS1DataBarOutput,
						ParamName.UPCE0,
						ParamName.UPCE1,
						ParamName.EAN13,
						ParamName.EAN8,
						ParamName.MSI,
						ParamName.GS1DataBarOmnidirectional,
						ParamName.GS1DataBarLimited,
						ParamName.GS1DataBarExpanded,
						ParamName.TriopticCode,
						ParamName.CodablockA,
						ParamName.CodablockF,
						ParamName.PDF417,
						ParamName.MacroPDF417,
						ParamName.MicroPDF417,
						ParamName.GS1CompositeCodes,
						ParamName.GS1Emulation,
						ParamName.TLC39,
						ParamName.QRCode,
						ParamName.Matrix,
						ParamName.MaxiCode,
						ParamName.AztecCode,
						ParamName.HanXinCode,
						ParamName.PostalCodes,
						ParamName.ChinaPost,
						ParamName.KoreaPost

				};
				adpSymbols.clear();
				for (ParamName name : names) {
					value = getReader().getBarcodeParam(name);
					if (value.getName() != ParamName.Unknown) {
						adpSymbols.addItem(value.getName(), value.getBoolean());
					}
				}
			} catch (Exception e) {
				ASRLog.e(TAG, e, "ERROR. $mChangeBarcodeMode.run() - Failed to change barcode mode");
			}

			runOnUiThread(new Runnable() {

				@Override
				public void run() {
					// Barcode Initialize Complete...
					lstSymbols.setAdapter(adpSymbols);
					adpSymbols.refreshAdapter();
					WaitDialog.hide();
					// Enable action widget
					enableActionWidgets(true);
				}

			});
		}

	};

	private Runnable mSaveSymbol = new Runnable() {

		@Override
		public void run() {
			int count = adpSymbols.getCount();
			ParamValue value;

			ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. Save Symbologies");

			for (int i = 0; i < count; i++) {
				value = adpSymbols.getParamValue(i);
				if (getReader().setBarcodeParam(value)) {
					ASRLog.d(TAG, ASRLog.DEBUG, "SAVE. %s : %s", value.getName(), value.getBoolean());
				} else {
					ASRLog.e(TAG, "ERROR. onStart() - Failed to set barcode param [%s]", value);
				}
			}

			runOnUiThread(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					// Enable action widget
					enableActionWidgets(true);
//					finish();
					AlertDialog.Builder builder = new AlertDialog.Builder(BarcodeOptionActivity.this);
					builder.setTitle("Success");
					builder.setMessage("Saved successfully")
							.setCancelable(false)
							.setPositiveButton("OK", null);

					AlertDialog dlg = builder.create();
					dlg.show();
					enableActionWidgets(true);
				}
			});
		}

	};

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

		if (getReader().isAvailableDefaultLinkProfile())
			return;

		Intent intent;
		ParamName p = adpSymbols.getItem(position);
		switch(p) {
		case Codabar:
//			intent = new Intent(this, CodabarSymbolActivity.class);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case Code39:
//			intent = new Intent(this, Code39SymbolActivity.class);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case I2of5:
		case NEC2of5:
//			intent = new Intent(this, I2of5SymbolActivity.class);
//			intent.putExtra("Symbol_Name", p);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case UPCA:
		case UPCE0:
		case UPCE1:
		case EAN8:
		case EAN13:
//			intent = new Intent(this, UpcEanSymbolActivity.class);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case Code93:
		case QRCode:
		case Matrix:
		case AztecCode:
//			intent = new Intent(this, Code93SymbolActivity.class);
//			intent.putExtra("Symbol_Name", p);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case R2of5:
		case A2of5:
		case CodablockA:
		case CodablockF:
		case PDF417:
		case MicroPDF417:
		case MaxiCode:
		case HanXinCode:
		case ChinaPost:
		case GS1128:
		case X2of5:
		case GS1DataBarExpanded:
//			intent = new Intent(this, LengthSymbolActivity.class);
//			intent.putExtra("Symbol_Name", p);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case Code128:
//			intent = new Intent(this, Code128SymbolActivity.class);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case Code11:
		case KoreaPost:
//			intent = new Intent(this, Code11SymbolActivity.class);
//			intent.putExtra("Symbol_Name", p);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case MSI:
//			intent = new Intent(this, MsiSymbolActivity.class);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		case GS1Emulation:
		case GS1CompositeCodes:
//			intent = new Intent(this, CompositeSymbolActivity.class);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_GS1_COMPOSITE);
			break;
		case PostalCodes:
//			intent = new Intent(this, PostalSymbolActivity.class);
//			startActivityForResult(intent, VIEW_BARCODE_OPTION_OTHERS);
			break;
		default:
			break;
		}
	}

	
}



